<?php
/*
Plugin Name: Tyres filter
Plugin URI: http://futurelab.co.nz
Version: 0.1
Author: Futurelab
Description: abc
*/

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Check if WooCommerce is active
 **/
if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins'))))
    return;


if (!class_exists('Tyres_Filter')) {

  /**
   * Main Tyres_Filter Class
   *
   * @class Tyres_Filter
   * @version 0.1
   */
  class Tyres_Filter {
    /**
     * @var string
     */
    public $version = '0.1';
    
    /**
     * @var string
     */
    public $text_domain = 'tyres-filter';
    

    /**
     * @var Tyres_Filter The single instance of the class
     * @since 0.1
     */
    protected static $_instance = null;
    
    /**
     * Main Tyres_Filter Instance
     *
     * Ensures only one instance of Tyres_Filter is loaded or can be loaded.
     *
     * @since 0.1
     * @static
     * @return Tyres_Filter - Main instance
     */
    public static function instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
    
    /**
     * Tyres_Filter Constructor.
     * @access public
     * @return Tyres_Filter
     */

    public function __construct() {
        $this->define_constants();
        $this->init();
    }
    
    public function define_constants(){
        $this->define( 'TYRES_FILTER_URL', plugin_dir_url( __FILE__ ) );
        $this->define( 'TYRES_FILTER_DIR', plugin_dir_path( __FILE__ ) );
        $this->define( 'TYRES_FILTER_BASENAME', plugin_basename(__FILE__) );
    }
    
    /**
     * Define constant if not already set
     * @param  string $name
     * @param  string|bool $value
     */
    private function define( $name, $value ) {
        if ( ! defined( $name ) ) {
            define( $name, $value );
        }
    }
    
    public function init(){
      add_action( 'init', array( $this, 'tyres_filter_init' ) );
      //enqueue script to wishlist page
      //add_action( 'yith_wcwl_before_wishlist_form', array($this, 'woo_print_wishlists_scripts') );
      //add print button after wishlists
      //add_action('yith_wcwl_after_wishlist_share', array($this, 'woo_print_button'),20);
      add_shortcode( 'tyres-filter', array($this, 'tyres_filter_shortcode') );
      add_action( 'woocommerce_before_shop_loop', array($this, 'tyres_filter_info'), 10 );
      //frontend ajax
      add_action( 'wp_ajax_ajax_get_attributes', array($this, 'ajax_get_attributes_callback') );  
      add_action( 'wp_ajax_nopriv_ajax_get_attributes', array($this, 'ajax_get_attributes_callback') );
      
      
    }
    
    public function tyres_filter_init() {
      if(!empty($_GET) && ! is_admin()){
        add_filter('loop_shop_post_in', array( $this, 'tyres_filter_query' ) );
      }
    }
  
    function tyres_filter_query($filtered_posts) {
      global $wpdb;
      $_chosen_attributes = array();
      
      if(!empty($_GET)){
        if(isset($_GET['width'])){
          $_chosen_attributes['width'] = $_GET['width'];
        }
        if(isset($_GET['height'])){
          $_chosen_attributes['height'] = $_GET['height'];
        }
        if(isset($_GET['rim'])){
          $_chosen_attributes['rim'] = $_GET['rim'];
        }
      }
     
      if ( sizeof( $_chosen_attributes ) > 0 ) {
        
        if(!empty($_chosen_attributes['width'])){
          $widthsql = "AND pm.`meta_key` = 'attribute_pa_width' 
                       AND pm.meta_value = '". $_chosen_attributes['width'] ."' ";
        }else{
          $widthsql = "";
        }
        
        if(!empty($_chosen_attributes['height'])){
          $heightsql = "AND pm2.`meta_key` = 'attribute_pa_height' 
                        AND pm2.meta_value = '". $_chosen_attributes['height'] ."' ";
        }else{
          $heightsql = "";
        }
        
        if(!empty($_chosen_attributes['rim'])){
          $rimsql = "AND pm3.`meta_key` = 'attribute_pa_rim' 
                     AND pm3.meta_value = '". $_chosen_attributes['rim'] ."' ";
        }else{
          $rimsql = "";
        }

        
        $querystr = "
          SELECT p.post_parent FROM wp_posts p 
          LEFT JOIN wp_postmeta pm ON p.id = pm.post_id 
          LEFT JOIN wp_postmeta pm2 ON pm.post_id = pm2.post_id 
          LEFT JOIN wp_postmeta pm3 ON pm.post_id = pm3.post_id 
          where p.`post_type` = 'product_variation' 
          ".$widthsql."
          ".$heightsql."
          ".$rimsql."
          group by p.post_parent
       ";
        $matched_products_result = $wpdb->get_results($querystr, OBJECT);
        
        if ( $matched_products_result ) {
          foreach ( $matched_products_result as $product ) {
            $matched_products[] = (int)$product->post_parent;
          }
        }

        // Filter the id's
        if ( 0 === sizeof( $filtered_posts ) ) {
          $filtered_posts = $matched_products;
        } else {
          $filtered_posts = array_intersect( $filtered_posts, $matched_products );
        }
        $filtered_posts[] = 0;
      }
       
      return (array) $filtered_posts;
    }
    
    /*
    * [tyres-filter]
    */
    function tyres_filter_shortcode() {
      $this -> tyres_filter_scripts();
      global $wpdb;
      $querystr = "
          SELECT pm.meta_key, pm.meta_value FROM wp_posts p 
          LEFT JOIN wp_postmeta pm ON p.id = pm.post_id 
          where p.`post_type` = 'product_variation' 
          AND pm.`meta_key` = 'attribute_pa_width' 
          AND pm.`meta_value` != '' 
          AND p.`post_status` = 'publish' GROUP BY pm.meta_value
       ";
     if (class_exists('Memcached')) {
       $m = new Memcached();
       $m->addServer('localhost', 11211);
       $key = md5('discount_tyres_attributes_width'); // Unique Words
       $cache_result = $m->get($key);
       if($cache_result){
         $attributes_width = $cache_result;
         //echo "cached";
       }else{
         $attributes_width = $wpdb->get_results($querystr, OBJECT);
         $m->set($key, $attributes_width, time() + 3600);
         //echo "set cache";
       }
     }else{
        $attributes_width = $wpdb->get_results($querystr, OBJECT);
        //echo "no cache";
     }
     
     //echo "<pre>".print_r($attributes_width,1);
      $curr_action = get_permalink( wc_get_page_id( 'shop' )  );
       ?>
      <h4 class="tyres_filter_title">Find a Tyre</h4>
      <form class="tyres_filter" action="<?php echo $curr_action; ?>" method="get" enctype='multipart/form-data'>

        <?php if ( empty( $attributes_width ) ) : ?>
          <p class="stock out-of-stock"><?php _e( 'This product is currently out of stock and unavailable.', 'woocommerce' ); ?></p>
        <?php else : ?>

          <table class="variations" cellspacing="0">
            <tbody>
                <tr>
                  <td class="label pa_width"><label for="pa_width">A</label></td>
                  <td class="value">
                      <select id="pa_width" class="" name="width" data-attribute_name="attribute_pa_width">

                        <?php echo '<option value="">Width</option>'; ?>

                        <?php
                        if ( ! empty( $attributes_width ) ) {
                          foreach ( $attributes_width as $option ) {

                            echo '<option value="' . esc_attr( $option->meta_value ) . '" >' . str_replace("-",".",$option->meta_value) . '</option>';
                          }
                        }
                        ?>
                      </select>
                  </td>
                </tr>
                <tr>
                  <td class="label pa_height"><label for="pa_height">B</label></td>
                  <td class="value">
                      <select id="pa_height" class="" name="height" data-attribute_name="attribute_pa_height" disabled>

                        <?php echo '<option value="">Height</option>'; ?>

                      </select>
                  </td>
                </tr>
                <tr>
                  <td class="label pa_rim"><label for="pa_rim">C</label></td>
                  <td class="value">
                      <select id="pa_rim" class="" name="rim" data-attribute_name="attribute_pa_rim" disabled>

                        <?php echo '<option value="">Rim</option>'; ?>

                      </select>
                  </td>
                </tr>
                <tr>
                  <td style="text-align: center; vertical-align: middle;"><img src="<?php echo TYRES_FILTER_URL; ?>/assets/images/ajax-loader.gif" class="ajax-loading" alt="loading" width="16" height="16" style="visibility:hidden"></td>
                  <td class="btn">
                      <input class="filter_btn" type="submit" value="FILTER">
                  </td>
                </tr>
            </tbody>
          </table>


        <?php endif; ?>

      </form>
       
    <?php
    
    }
    
    function tyres_filter_info() {
      if(!empty($_GET)){
      ?>
        <div class="tyres_filter_info">
          <span class="tyres_filter_info_title">You searched for tyres this size:</span><br>
          <span class="tyres_filter_info_size">
              <?php if(isset($_GET['width'])){?>
              width <strong><?php echo (empty($_GET['width'])) ? 'Any Width': str_replace("-",".",$_GET['width']) ; ?></strong>
              <?php }?>
              <?php if(isset($_GET['height'])){?>
              , ratio <strong><?php echo (empty($_GET['height'])) ? 'Any Height': str_replace("-",".",$_GET['height']) ; ?></strong>
              <?php }?>
              <?php if(isset($_GET['rim'])){?>
              , rim size <strong><?php echo (empty($_GET['rim'])) ? 'Any Rim': str_replace("-",".",$_GET['rim']) ; ?></strong>
              <?php }?>
          </span>
        </div>
      <?php
      }
    }
    
    
    function tyres_filter_scripts() {
      wp_enqueue_style( 'tyres-filter-style', TYRES_FILTER_URL . 'assets/css/style.css', array( ),$this->version );
      // Register the script
      wp_register_script( 'tyres-filter-js', TYRES_FILTER_URL . 'assets/js/tyres-filter.js', array( 'jquery' ),$this->version );

      // Localize the script with new data
      $translation_array = array(
        'ajax' => admin_url( 'admin-ajax.php' )
      );
      wp_localize_script( 'tyres-filter-js', 'tyres_filter', $translation_array );

      // Enqueued script with localized data.
      wp_enqueue_script( 'tyres-filter-js' );
    }
    

    function ajax_get_attributes_callback() {
      if ( isset( $_POST['attribute'] ) ) {
        if($_POST['attribute'] == "Height"){
          $width = $_POST['value'];
          echo '<option value="">'. $_POST['attribute'] .'</option>';
          global $wpdb;
          $querystr = "
            SELECT pm2.meta_key, pm2.meta_value FROM wp_posts p 
            LEFT JOIN wp_postmeta pm ON p.id = pm.post_id 
            LEFT JOIN wp_postmeta pm2 ON pm.post_id = pm2.post_id 
            where p.`post_type` = 'product_variation' 
            AND pm.`meta_key` = 'attribute_pa_width' 
            AND pm.meta_value = '$width' 
            AND pm2.`meta_key` = 'attribute_pa_height' 
            AND pm2.meta_value != ''
            group by pm2.meta_value
          ";

          $attributes_height = $wpdb->get_results($querystr, OBJECT);
          //echo "<pre>".print_r($attributes_height,1);
          if ( ! empty( $attributes_height ) ) {
            foreach ( $attributes_height as $option ) {
              
              echo '<option value="' . esc_attr( $option->meta_value ) . '" >' . str_replace("-",".",$option->meta_value) . '</option>';
            }
          }
        }
        if($_POST['attribute'] == "Rim"){
          $width = $_POST['value'];
          $height = $_POST['value2'];
          if(!empty($height)){
            $heightsql = "AND pm2.`meta_key` = 'attribute_pa_height' 
                          AND pm2.meta_value = '$height' ";
          }else{
            $heightsql = "";
          }
          echo '<option value="">'. $_POST['attribute'] .'</option>';
          global $wpdb;
          $querystr = "
            SELECT pm3.meta_key, pm3.meta_value FROM wp_posts p 
            LEFT JOIN wp_postmeta pm ON p.id = pm.post_id 
            LEFT JOIN wp_postmeta pm2 ON pm.post_id = pm2.post_id 
            LEFT JOIN wp_postmeta pm3 ON pm.post_id = pm3.post_id 
            where p.`post_type` = 'product_variation' 
            AND pm.`meta_key` = 'attribute_pa_width' 
            AND pm.meta_value = '$width' 
            ".$heightsql."
            AND pm3.`meta_key` = 'attribute_pa_rim' 
            AND pm3.meta_value != ''
            group by pm3.meta_value
          ";

          $attributes_height = $wpdb->get_results($querystr, OBJECT);
          if ( ! empty( $attributes_height ) ) {
            foreach ( $attributes_height as $option ) {
              
              echo '<option value="' . esc_attr( $option->meta_value ) . '" >' . str_replace("-",".",$option->meta_value) . '</option>';
            }
          }
        }
        
      }
      wp_die(); // this is required to terminate immediately and return a proper response
    }
  }
  
  /**
  * Returns the main instance of Tyres_Filter to prevent the need to use globals.
  *
  * @since  0.1
  * @return Tyres_Filter
  */
  function tyres_filter_run() {
      return Tyres_Filter::instance();
  }

  // Global for backwards compatibility.
  global $tyres_filter;

  $tyres_filter = tyres_filter_run();
}

?>
